Stan Math Library: stan/math/prim/scal/prob/neg_binomial_cdf.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

neg_binomial_cdf.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_NEG_BINOMIAL_CDF_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_NEG_BINOMIAL_CDF_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/meta/include_summand.hpp>
8 #include <stan/math/prim/scal/fun/constants.hpp>
9 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
10 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
11 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
12 #include <stan/math/prim/scal/err/check_not_nan.hpp>
13 #include <stan/math/prim/scal/fun/value_of.hpp>
14 #include <stan/math/prim/scal/fun/digamma.hpp>
15 #include <stan/math/prim/scal/fun/inc_beta.hpp>
16 #include <stan/math/prim/scal/fun/inc_beta_dda.hpp>
17 #include <stan/math/prim/scal/fun/inc_beta_ddb.hpp>
18 #include <stan/math/prim/scal/fun/inc_beta_ddz.hpp>
19 #include <cmath>
20 #include <limits>
21
22 namespace stan {
23 namespace math {
24
25 // Negative Binomial CDF
26 template <typename T_n, typename T_shape,
27 typename T_inv_scale>
28 typename return_type<T_shape, T_inv_scale>::type
29 neg_binomial_cdf(const T_n& n, const T_shape& alpha,
30 const T_inv_scale& beta) {
31 static const char* function("stan::math::neg_binomial_cdf");
32 typedef typename stan::partials_return_type<T_n, T_shape,
33 T_inv_scale>::type
34 T_partials_return;
35
36 using stan::math::check_positive_finite;
37 using stan::math::check_consistent_sizes;
38
39 // Ensure non-zero arugment lengths
40 if (!(stan::length(n) && stan::length(alpha) && stan::length(beta)))
41 return 1.0;
42
43 T_partials_return P(1.0);
44
45 // Validate arguments
46 check_positive_finite(function, "Shape parameter", alpha);
47 check_positive_finite(function, "Inverse scale parameter", beta);
48 check_consistent_sizes(function,
49 "Failures variable", n,
50 "Shape parameter", alpha,
51 "Inverse scale parameter", beta);
52
53 // Wrap arguments in vector views
54 VectorView<const T_n> n_vec(n);
55 VectorView<const T_shape> alpha_vec(alpha);
56 VectorView<const T_inv_scale> beta_vec(beta);
57 size_t size = max_size(n, alpha, beta);
58
59 // Compute vectorized CDF and gradient
60 using stan::math::value_of;
61 using stan::math::inc_beta;
62 using stan::math::inc_beta_ddz;
63 using stan::math::inc_beta_dda;
64 using stan::math::digamma;
65
66 OperandsAndPartials<T_shape, T_inv_scale>
67 operands_and_partials(alpha, beta);
68
69 // Explicit return for extreme values
70 // The gradients are technically ill-defined, but treated as zero
71 for (size_t i = 0; i < stan::length(n); i++) {
72 if (value_of(n_vec[i]) < 0)
73 return operands_and_partials.value(0.0);
74 }
75
76 // Cache a few expensive function calls if alpha is a parameter
77 VectorBuilder<!is_constant_struct<T_shape>::value,
78 T_partials_return, T_shape>
79 digamma_alpha_vec(stan::length(alpha));
80
81 VectorBuilder<!is_constant_struct<T_shape>::value,
82 T_partials_return, T_shape>
83 digamma_sum_vec(stan::length(alpha));
84
85 if (!is_constant_struct<T_shape>::value) {
86 for (size_t i = 0; i < stan::length(alpha); i++) {
87 const T_partials_return n_dbl = value_of(n_vec[i]);
88 const T_partials_return alpha_dbl = value_of(alpha_vec[i]);
89
90 digamma_alpha_vec[i] = digamma(alpha_dbl);
91 digamma_sum_vec[i] = digamma(n_dbl + alpha_dbl + 1);
92 }
93 }
94
95 for (size_t i = 0; i < size; i++) {
96 // Explicit results for extreme values
97 // The gradients are technically ill-defined, but treated as zero
98 if (value_of(n_vec[i]) == std::numeric_limits<int>::max())
99 return operands_and_partials.value(1.0);
100
101 const T_partials_return n_dbl = value_of(n_vec[i]);
102 const T_partials_return alpha_dbl = value_of(alpha_vec[i]);
103 const T_partials_return beta_dbl = value_of(beta_vec[i]);
104
105 const T_partials_return p_dbl = beta_dbl / (1.0 + beta_dbl);
106 const T_partials_return d_dbl = 1.0 / ( (1.0 + beta_dbl)
107 * (1.0 + beta_dbl) );
108
109 const T_partials_return P_i =
110 inc_beta(alpha_dbl, n_dbl + 1.0, p_dbl);
111
112 P *= P_i;
113
114 if (!is_constant_struct<T_shape>::value) {
115 operands_and_partials.d_x1[i]
116 += inc_beta_dda(alpha_dbl, n_dbl + 1, p_dbl,
117 digamma_alpha_vec[i],
118 digamma_sum_vec[i]) / P_i;
119 }
120
121 if (!is_constant_struct<T_inv_scale>::value)
122 operands_and_partials.d_x2[i] +=
123 inc_beta_ddz(alpha_dbl, n_dbl + 1.0, p_dbl) * d_dbl / P_i;
124 }
125
126 if (!is_constant_struct<T_shape>::value) {
127 for (size_t i = 0; i < stan::length(alpha); ++i)
128 operands_and_partials.d_x1[i] *= P;
129 }
130
131 if (!is_constant_struct<T_inv_scale>::value) {
132 for (size_t i = 0; i < stan::length(beta); ++i)
133 operands_and_partials.d_x2[i] *= P;
134 }
135
136 return operands_and_partials.value(P);
137 }
138
139 } // prob
140 } // stan
141 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::neg_binomial_cdf
return_type< T_shape, T_inv_scale >::type neg_binomial_cdf(const T_n &n, const T_shape &alpha, const T_inv_scale &beta)
Definition: neg_binomial_cdf.hpp:29
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::inc_beta_dda
T inc_beta_dda(T a, T b, T z, T digamma_a, T digamma_ab)
Returns the partial derivative of the regularized incomplete beta function, I_{z}(a, b) with respect to a.
Definition: inc_beta_dda.hpp:39
inc_beta_ddz.hpp
digamma.hpp
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::inc_beta_ddz
T inc_beta_ddz(T a, T b, T z)
Returns the partial derivative of the regularized incomplete beta function, I_{z}(a, b) with respect to z.
Definition: inc_beta_ddz.hpp:27
inc_beta_ddb.hpp
include_summand.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::inc_beta
fvar< T > inc_beta(const fvar< T > &a, const fvar< T > &b, const fvar< T > &x)
Definition: inc_beta.hpp:20
stan::partials_return_type
Definition: partials_return_type.hpp:16
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
stan::math::max
int max(const std::vector< int > &x)
Returns the maximum coefficient in the specified column vector.
Definition: max.hpp:21
partials_return_type.hpp
OperandsAndPartials.hpp
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::size
int size(const std::vector< T > &x)
Return the size of the specified standard vector.
Definition: size.hpp:17
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
inc_beta.hpp
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16
inc_beta_dda.hpp

[ Stan Home Page ] © 2011–2016, Stan Development Team.